/**
 -------------------------------------------------------------------------------------------------------------------------------
This file provides boilerplate templates for interfacing with the GUI Composer framework.
For further information, see the available options under the Help main menu in the Designer.
-------------------------------------------------------------------------------------------------------------------------------
**/

import { GcUtils } from './components/@ti/gc-core-assets/lib/GcUtils';
import { GcConsole } from './components/@ti/gc-core-assets/lib/GcConsole';
import { bindingRegistry } from './components/@ti/gc-core-databind/lib/CoreDatabind';
import { GcWidget } from './components/@ti/gc-widget-base/lib/GcWidget';
import { ActionRegistry } from './components/@ti/gc-widget-menu/lib/ActionRegistry';

const console = new GcConsole('myapp'); // creates a console instance with name 'myapp'
GcConsole.setLevel('myapp', 4);         // enable console output for myapp console instance
console.info('index.js is loaded...');

var GPN_JSON;
var count;

const init = async () => {
    // Add menubar product-name-clicked event listener
    GcWidget.querySelector('gc-widget-menubar').then(menubar => {
        menubar.addEventListener('product-name-clicked', () => window.open('https://dev.ti.com/gc', 'gc'));
    });
    
    //Plotly
    var trace = {
        y: [],
        x: []
    };
    
    var trace2 = {
        y: [],
        x: [],
        yaxis: 'y2'
    }
        
    var data1 = [trace];
    var data2 = [trace, trace2];

    var layoutDC = { 
        font: {size: 10},
        autosize: true,
        margin: {
        l: 40,
        r: 40,
        b: 30,
        t: 30
        },
        xaxis: {
            title: 'V<sub>OUT</sub> (V)'
        },
        yaxis: {
            title: 'I<sub>LOAD</sub> (A)'
        }
    };
        
    var layoutTransient = { 
        font: {size: 10},
        autosize: true,
        margin: {
        l: 40,
        r: 40,
        b: 30,
        t: 30
        },
        xaxis: {
            title: 't (s)'
        },
        yaxis: {
            title: 'I<sub>LOAD</sub> (A)'
        },
        yaxis2: {
            title: 'V<sub>OUT</sub> (V)',
            side: 'right'
        }
    };
    
    Plotly.newPlot("dcPlot", data1, layoutDC);
    Plotly.newPlot("transientPlot", data2, layoutTransient);
    
    //wait until all widgets are loaded
    await GcWidget.querySelectorAll('gc-widget-label');
    await GcWidget.querySelectorAll('gc-widget-select');
    await GcWidget.querySelectorAll('gc-widget-button');
    await GcWidget.querySelectorAll('gc-widget-container');
    await GcWidget.querySelectorAll('gc-widget-input');
    
    //asynchronous function to load the list of high speed opamps from the TI API into the selection dropdown. 
    //TODO: add screen pause to make sure user cannot interact with elements until loaded
    const response = await fetch('https://www.ti.com/selectionmodel/api/gpn/result-list?destinationId=0&destinationType=GPT&mode=parametric&familyId=72&locale=en-US');
    const myJson = await response.json(); //extract JSON from the http response
    
    GPN_JSON = myJson;
    console.log("test JSON global");
    console.log(GPN_JSON);
    
    var gpnList = [];
    
    
    
    //loop through each entry under "results" array from JSON object and add its GPN to gpnList array
    myJson.results.forEach((element) => {
        gpnList.push(element.genericPartNumber);
    }) 
    
    console.log("List of opamps (unsorted): "+gpnList);
    
    //sort the opamps based on GBW
    const gpnListWithGBW = [];
    const indexMap = new Map();
    for(var i = 0; i < gpnList.length; i++){
        indexMap[gpnList[i]] = i;
        //check if GBW parameter is present
        const GPNObject = GPN_JSON.results[i]; //get the gpn object from the API JSON
        const gbwIndex = GPNObject.paramList.findIndex(params => params.uniqueIdentifier === "23typ");
        if(GPNObject.paramList[gbwIndex] != undefined) {
            const gbwValue = GPNObject.paramList[gbwIndex].value.M; 
            gpnListWithGBW.push(gpnList[i] + " " + gbwValue);
        }
    }
    console.log("gpnList with GBW values = "+gpnListWithGBW);
    
    // Extract and convert the numeric part to numbers
    const sortedArr = gpnListWithGBW.sort((a, b) => {
        const numA = parseInt(a.split(" ")[1]);
        const numB = parseInt(b.split(" ")[1]);
        return numA - numB;
    });
    
    console.log("gpnList with GBW values (sorted) = "+gpnListWithGBW);
    
    count = 0;
    for (let i = 0; i < gpnListWithGBW.length; i++) {
        var gpn = gpnListWithGBW[i].split(" ", 2);
        let option = gpn[0];
        
        //Manually add the first device, to compensate for the delay in API call
        if(option == "OPA2834") continue;
        
        //Block to get the GBW (typ), architechture, Vs_max, rail-to-rail spec for each opamp
        const GPNObject = GPN_JSON.results[indexMap[option]]; //get the gpn object from the API JSON
        const gbwIndex = GPNObject.paramList.findIndex(params => params.uniqueIdentifier === "23typ"); 
        const archIndex = GPNObject.paramList.findIndex(params => params.uniqueIdentifier === "89"); 
        const maxSupplyIndex = GPNObject.paramList.findIndex(params => params.uniqueIdentifier === "1261max");
        const railToRailIndex = GPNObject.paramList.findIndex(params => params.uniqueIdentifier === "78"); //array with 2 values, first for IN and second for OUT
        
        if(GPNObject.paramList[gbwIndex] != undefined && GPNObject.paramList[archIndex] != undefined && GPNObject.paramList[maxSupplyIndex] != undefined && GPNObject.paramList[railToRailIndex] != undefined) {
            const gbwValue = GPNObject.paramList[gbwIndex].value.M; 
            const archValue = GPNObject.paramList[archIndex].value.base;
            const maxSupplyValue = GPNObject.paramList[maxSupplyIndex].value.base[0];
            const railToRailValue = GPNObject.paramList[railToRailIndex].value.base;
            
            //only include Voltage FB or FET / CMOS Input opamps
            if((archValue[0] === "Voltage FB" || (archValue[0] === "FET / CMOS Input" && archValue[1] === "Voltage FB") || (archValue[1] === "FET / CMOS Input" && archValue[0] === "Voltage FB")) && (maxSupplyValue <= 15) && (railToRailValue.length != 1 && (railToRailValue[0] === "In" || railToRailValue[0] === "In to V-") && railToRailValue[1] === "Out")){
                // console.log("Architechture = "+archValue);
                // console.log("GBW for opamp ("+gpnList[i]+") = "+gbwValue);
                // console.log("maxSupplyValue = "+maxSupplyValue);
                // console.log("Rail to rail = "+railToRailValue);
                
                let el = document.createElement("option");
                el.textContent = option + " (" + gbwValue + " MHz)";
                el.value = option + " (" + gbwValue + ")";
                opampSelectDropDown.appendChild(el);
                count++;
            }
        }
    }
    
    console.log("Total count = "+ count);
    
    //Set default opamp as TLV365 which is at 17th index starting from 0
    opampSelectDropDown.selectedIndex = 17;
    updateOpampSpecsTable();
};
document.readyState === 'complete' ? init() : document.addEventListener('DOMContentLoaded', init);


//Script starts

    //User inputs
    var maxFreq;
    var peakCurrent; 
    var maxLoadCurrent;
    var minLoadCurrent;
    var minVout;
    var maxVout;
    var negSupplyVolts;
    var posSupplyVolts;
    var rShunt;
    var opamp;
    var tolerance;
    var minDutyCycle;
    
    //variables to be populated by API call 
    var gbwTyp;
    var slewRateTyp;
    var inputBiasCurrMax;
    var inputOffsetVoltsMax;
    var supplyVoltsMax;
    var supplyVoltsMin;
    var outSwingHeadRoomToPos;
    var outSwingHeadRoomToNeg;
    var railToRail;
    var minSpecGain;
    
    const sineWaveRadio = document.getElementById("sineWaveRadio"); 
    const switchingWaveRadio = document.getElementById("switchingWaveRadio"); 
    var dcLevelInput = document.getElementById("dcLevelInput");
    var amplitudeInput = document.getElementById("amplitudeInput");
    var frequencyInput = document.getElementById("frequencyInput");
    var minimumDutyCycleInput = document.getElementById("minimumDutyCycleInput");
    var minimumDutyCycleLabel = document.getElementById("minimumDutyCycleLabel");
    var opampSelectDropDown = document.getElementById("opampSelectDropDown");
    var positiveSupplyVoltageInput = document.getElementById("positiveSupplyVoltageInput");
    var negativeSupplyVoltageInput = document.getElementById("negativeSupplyVoltageInput");
    var maximumVoutInput = document.getElementById("maximumVoutInput");
    var minimumVoutInput = document.getElementById("minimumVoutInput");
    var shuntResistanceInput = document.getElementById("shuntResistanceInput");
    var toleranceSelectDropDown = document.getElementById("toleranceSelectDropDown");
    var designCircuitButton = document.getElementById("designCircuitButton");
    var resetButton = document.getElementById("resetButton");
    var loader = document.getElementById("loader");
    var opampSpecificationsContainer = document.getElementById("opampSpecificationsContainer");
    var minSupplyVoltageField = document.getElementById("minSupplyVoltageField");
    var maxSupplyVoltageField = document.getElementById("maxSupplyVoltageField");
    var gbwTypField = document.getElementById("gbwTypField");
    var inputOffsetVoltageMaxField = document.getElementById("inputOffsetVoltageMaxField");
    var railToRailField = document.getElementById("railToRailField");
    var slewRateField = document.getElementById("slewRateField");
    var productLink = document.getElementById("productLink");
    var e2eForumLink = document.getElementById("e2eForumLink");
    var r1Field = document.getElementById("r1Field");
    var r2Field = document.getElementById("r2Field"); 
    var r3Field = document.getElementById("r3Field");
    var r4Field = document.getElementById("r4Field");
    var closedLoopGainField = document.getElementById("closedLoopGainField");
    var shuntPowerLossField = document.getElementById("shuntPowerLossField");
    var settlingTimeField = document.getElementById("settlingTimeField");
    var riseTimeField = document.getElementById("riseTimeField");
    var vrefField = document.getElementById("vrefField");
    var waveFormImages = document.getElementById("waveFormImages");
    var circuitDiagramImages = document.getElementById("circuitDiagramImages");
    var equationField = document.getElementById("equationField");
    var equationFieldNew = document.getElementById("equationFieldNew");
    var opampField = document.getElementById("opampField");
    var warningsContainer = document.getElementById("warningsContainer");
    var errorInCurrentUnit = document.getElementById("errorInCurrentUnit");
    var r1UserValue;
    var r2UserValue;
    var r3UserValue;
    var r4UserValue;
    
    minimumDutyCycleInput.style.display = "none";
    minimumDutyCycleLabel.style.display = "none";
    
    //Setting default input values
    sineWaveRadio.checked = false;
    switchingWaveRadio.checked = true;
    minimumDutyCycleInput.style.display = 'block'; 
    minimumDutyCycleLabel.style.display = 'block';
    dcLevelLabel.style.display = 'none';
    dcLevelInput.style.display = 'none';
    ampereLabel1.label = "%";
    waveFormImages.selectedIndex = 1;
    toleranceSelectDropDown.value = '0.1';
    
    //API call
    async function getJsonResponse(gpn) {
        var gpnExtracted = gpn.split(" ",1); //extract the gpn from the selected opamp in drop down list
        const response = await fetch('https://www.ti.com/selectionmodel/api/opn/result-list?gpn=' + gpnExtracted[0] + '&locale=en-US');
        const jsonResponse = await response.json(); //extract JSON from the http response
        return jsonResponse;
    }
    
    //Add event listeners 
    sineWaveRadio.addEventListener('change', function() { if (sineWaveRadio.checked) { 
        minimumDutyCycleInput.style.display = 'none';
        minimumDutyCycleLabel.style.display = 'none';
        dcLevelLabel.style.display = 'block';
        dcLevelInput.style.display = 'block';
        ampereLabel1.label = "A";
        waveFormImages.selectedIndex = 0;
    } }); 
    
    switchingWaveRadio.addEventListener('change', function() { if (switchingWaveRadio.checked) { 
        minimumDutyCycleInput.style.display = 'block'; 
        minimumDutyCycleLabel.style.display = 'block';
        dcLevelLabel.style.display = 'none';
        dcLevelInput.style.display = 'none';
        ampereLabel1.label = "%";
        waveFormImages.selectedIndex = 1;
    } });
    
    r2Field.addEventListener("input", changeR2);
    r4Field.addEventListener("input", changeR4);
    
    e2eForumLink.href = "https://e2e.ti.com/support/amplifiers-group/amplifiers/f/amplifiers-forum/1270018/faq-h-frequently-asked-questions---high-speed-amplifiers";
    
    //Update the opamp specs table when user first opens the tool
    //updateSpecsForDefaultOpamp();
    // async function updateSpecsForDefaultOpamp() {
    //     //API call to get opamp specs
    //     var gpn = opampSelectDropDown.value;
    //     console.log("selected opamp: "+gpn);
    //     var gpnExtracted = gpn.split(" ",1); //extract the gpn from the selected opamp in drop down list
    //     const myJson = await getJsonResponse(gpnExtracted[0]);
        
    //     //show the parameters on opamp specs table
    //     slewRateTyp = myJson[0].paramList.find(param => param.name === "Slew ratetyp").value.base[0];
    //     minSpecGain = myJson[0].paramList.find(param => param.name === "Acl, min spec gain").value.base[0];
    //     inputOffsetVoltsMax = myJson[0].paramList.find(param => param.name === "Vos (offset voltage at 25°C)max").value.base[0];
    //     supplyVoltsMax = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)max").value.base[0];
    //     supplyVoltsMin = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)min").value.base[0];
    //     var arr = myJson[0].paramList.find(param => param.name === "Rail-to-rail").value.base;
    //     if(arr.length == 2) railToRailField.label = arr[0]+", "+arr[1];
    //     else if(arr[0] == "No") railToRailField.label = "No";
    //     console.log("Rail to rail = "+arr);
    
    //     //update opamps specs container and its heading
    //     opampSpecificationsContainer.heading = gpnExtracted + " specifications";
    //     slewRateField.label = slewRateTyp;
    //     minSupplyVoltageField.label = supplyVoltsMin;
    //     maxSupplyVoltageField.label = supplyVoltsMax;
    //     minSpecGainField.label = minSpecGain;
    //     console.log("Vos(us) = "+(inputOffsetVoltsMax * 1000000));
    //     if(inputOffsetVoltsMax * 1000000 > 500){
    //         inputOffsetVoltageMaxField.label = inputOffsetVoltsMax * 1000;
    //         microVoltageLabel1.label = "mV";
    //     }
    //     else{
    //         inputOffsetVoltageMaxField.label = inputOffsetVoltsMax * 1000000;
    //         microVoltageLabel1.label = "μV";
    //     }
        
    //     //update links
    //     productLink.href = "https://ti.com/product/" + gpnExtracted;
        
    //     //Show opamp name on diagram
    //     opampField.label = gpnExtracted;
    // }
    
    async function updateOpampSpecsTable() {
        console.log("Opamp changed!");
        console.log("Selected index = "+opampSelectDropDown.selectedIndex);
        //API call to get opamp specs
        var gpn = opampSelectDropDown.value;
        console.log("selected opamp: "+gpn);
        var gpnExtracted = gpn.split(" ",1); //extract the gpn from the selected opamp in drop down list
        const myJson = await getJsonResponse(gpnExtracted[0]);
        
        //show the parameters on opamp specs table, if the parameter exists
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Slew ratetyp") != undefined){
            slewRateTyp = myJson[0].paramList.find(param => param.name === "Slew ratetyp").value.base[0];
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
        
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Acl, min spec gain") != undefined){
            minSpecGain = myJson[0].paramList.find(param => param.name === "Acl, min spec gain").value.base[0];
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
        
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Vos (offset voltage at 25°C)max")){
            inputOffsetVoltsMax = myJson[0].paramList.find(param => param.name === "Vos (offset voltage at 25°C)max").value.base[0];
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
        
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)max")) {
            supplyVoltsMax = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)max").value.base[0];
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
        
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)min")){
            supplyVoltsMin = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)min").value.base[0];
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
        
        if(myJson[0] != undefined && myJson[0].paramList != undefined && myJson[0].paramList.find(param => param.name === "Rail-to-rail")) {
            var arr = myJson[0].paramList.find(param => param.name === "Rail-to-rail").value.base;
            if(arr.length == 2) railToRailField.label = arr[0]+", "+arr[1];
            else if(arr[0] == "No") railToRailField.label = "No";
            console.log("Rail to rail = "+arr);
        }
        else{
            alert("The selected OpAmp does not include all the parameters needed for computing the design. Please select a different device.");
            opampSelectDropDown.selectedIndex = 0;
            return;
        }
    
        //update opamps specs container and its heading
        opampSpecificationsContainer.heading = gpnExtracted + " specifications";
        slewRateField.label = slewRateTyp;
        minSupplyVoltageField.label = supplyVoltsMin;
        maxSupplyVoltageField.label = supplyVoltsMax;
        minSpecGainField.label = minSpecGain;
        console.log("Vos(us) = "+(inputOffsetVoltsMax * 1000000));
        if(inputOffsetVoltsMax * 1000000 > 500){
            inputOffsetVoltageMaxField.label = inputOffsetVoltsMax * 1000;
            microVoltageLabel1.label = "mV";
        }
        else{
            inputOffsetVoltageMaxField.label = inputOffsetVoltsMax * 1000000;
            microVoltageLabel1.label = "μV";
        }
        
        //Show opamp name on diagram
        opampField.label = gpnExtracted;
        
        //update links
        productLink.href = "https://ti.com/product/" + gpnExtracted;
    }
    
    //Event listeners for updating opamp specs table
    opampSelectDropDown.addEventListener('change', updateOpampSpecsTable);
    
    //Disable the R2 and R4 inputs fields at the start
    r2Field.disabled = true;
    r4Field.disabled = true;
    
    //Disable loader at start
    loader.style.display = "none";
    
    //function to take convert inputs based on user input
    function convertInputs(input) {
        //if no input given or wrong input show alert (wrong input = 1kk, 10kn, etc.)
        if(input == undefined || isNaN(Number(input.slice(0, -1)))){
            return -1;    
        }
        
        //if "n" "k" or anything is used at the end of number, eg. 1k, 20n, etc.
        if(isNaN(Number(input))){
            switch(input[input.length - 1]) {
            case 'n':
                input = Number(input.slice(0, -1))/1e9;
                break;
            case 'u':
                input = Number(input.slice(0, -1))/1e6;
                break;
            case 'm':
                input = Number(input.slice(0, -1))/1e3;
                break;
            case 'k':
                input = Number(input.slice(0, -1)) * 1e3;
                break;
            case 'M':
                input = Number(input.slice(0, -1)) * 1e6;
                break;
            case 'G':
                input = Number(input.slice(0, -1)) * 1e9;
                break;
            default:
                console.log("Frequency read = " + input);
                return -1;
            }
        }
        console.log("Frequency read = "+input);
        return Number(input);
    }

    //Storing input variables
    function takeInputs(){
        maxFreq = convertInputs(frequencyInput.value);
        
        peakCurrent = amplitudeInput.value;
        if(switchingWaveRadio.checked) {
            minLoadCurrent = 0;
            maxLoadCurrent = amplitudeInput.value;
        }
        else {
            minLoadCurrent = dcLevelInput.value - peakCurrent;
            maxLoadCurrent = dcLevelInput.value + peakCurrent;
        }
        minVout = Number(minimumVoutInput.value);
        maxVout = Number(maximumVoutInput.value);
        negSupplyVolts = negativeSupplyVoltageInput.value;
        
        if(negSupplyVolts == 0){
            circuitDiagramImages.selectedIndex = 1;
        }
        
        else{
            circuitDiagramImages.selectedIndex = 0;
        }
        
        posSupplyVolts = positiveSupplyVoltageInput.value;
        rShunt = convertInputs(shuntResistanceInput.value);
        opamp = opampSelectDropDown.value;
        tolerance = toleranceSelectDropDown.value;
        minDutyCycle = minimumDutyCycleInput.value;
    }
    
    function createNode(suggestionText, textColor) {
        const node = document.createElement("li");
        const textNode = document.createTextNode(suggestionText);
        node.appendChild(textNode);
        node.style.fontSize ="12px";
        node.style.color = textColor;
        warningsContainer.appendChild(node);
    }
    
    function createWarnings(shuntPower, bw, supplyVoltsMax, supplyVoltsMin, settlingTime, minSpecGain, closedLoopGain, isOpampGBWLimited, r2UserValue, r4UserValue){
        const tiRed = "#CC0000";
        
        //Empty the warnings container
        while (warningsContainer.firstChild) {
            warningsContainer.firstChild.remove();
        }
        
        //Add default suggestions for Min required GBW and slew rate
        var lastOpampInList = opampSelectDropDown.options[count].value;
        var arr = lastOpampInList.split(" ");
        var lastOpampGpn = arr[0]; 
        var stage2Opamp = "TLV365 (50 MHz)";
        var lastOpampGBW = arr[1].slice(1);
        lastOpampGBW = lastOpampGBW.slice(0, lastOpampGBW.length - 1);
        lastOpampGBW = Number(lastOpampGBW); //GBW is in MHz
        
        var minGBWForSine = (maxFreq * (closedLoopGain + 1)) / (0.142);
        minGBWForSine = minGBWForSine/1e6;
        if(minGBWForSine < 1) minGBWForSine = minGBWForSine.toFixed(2);
        else minGBWForSine = Math.round(minGBWForSine);
        var minSlewRate = (Math.PI) * (maxVout - minVout) * maxFreq / 1e6; //1e6 to convert into V/us
        minSlewRate = (minSlewRate < 1) ? minSlewRate.toFixed(2) : Math.round(minSlewRate);
        if(sineWaveRadio.checked && minGBWForSine > 300) {
            //suggest a 2 stage implementation
            var AclMaxStage1 = (0.142 * lastOpampGBW * 1e6)/maxFreq - 1;
            var AclMaxStage2 = closedLoopGain/AclMaxStage1;
            if(AclMaxStage2 < 1) AclMaxStage2 = AclMaxStage2.toFixed(2);
            else AclMaxStage2 = AclMaxStage2.toFixed(1);
            createNode("The required gain bandwidth product (GBW) for the design is " + minGBWForSine + " MHz, and the required slew rate is " + 
            minSlewRate + " V/µs. Please consider using a two stage implementation with maximum closed loop gain in the first stage. Use " + lastOpampGpn + 
            " (" + lastOpampGBW + " MHz) in a gain of " + AclMaxStage1.toFixed(1) + " V/V followed by " + stage2Opamp + " in a gain of " + 
            AclMaxStage2 + " V/V, to achieve a total closed loop gain of " + Math.round(closedLoopGain) + " V/V.", tiRed);
        }
        
        //console.log("Max gbw of opamp from the list = "+lastOpampGBW);
        var minGBWForSwitching = (5 * (closedLoopGain + 1) * maxFreq) / (2 * Math.PI * 0.3 * (minDutyCycle/100) * 1e6); 
        if(minGBWForSwitching < 1) minGBWForSwitching = minGBWForSwitching.toFixed(2);
        else minGBWForSwitching = Math.round(minGBWForSwitching);
        if(switchingWaveRadio.checked && minGBWForSwitching > 300) {
            // if(minGBW <= 300) {
            //     createNode("The required gain bandwidth product (GBW) for the design is " + minGBW + " MHz.", black);
            // }
            
            //suggest a 2 stage implementation
            var AclMaxStage1 = (2 * Math.PI * 0.3 * (minDutyCycle/100) * lastOpampGBW * 1e6)/(5 * maxFreq) - 1; //calculated from the relation Tsettle (or 5*tau) <= 30% of T_ON
            var AclMaxStage2 = closedLoopGain / AclMaxStage1; //To get a total gain of closedLoopGain = AclMaxStage1 * AclMaxStage2
            if(AclMaxStage2 < 1) AclMaxStage2 = AclMaxStage2.toFixed(2);
            else AclMaxStage2 = AclMaxStage2.toFixed(1);
            createNode("The required gain bandwidth product (GBW) for the design is " + Math.round(minGBWForSwitching) + " MHz. Please consider using a two stage implementation with maximum closed loop gain in the first stage. Use " + lastOpampGpn + 
            " (" + lastOpampGBW + " MHz) in a gain of " + AclMaxStage1.toFixed(1) + " V/V followed by " + stage2Opamp + " in a gain of " + 
            AclMaxStage2 + " V/V, to achieve a total closed loop gain of " + Math.round(closedLoopGain) + " V/V.", tiRed);
            
        }
        
        //Shunt power warning
        if(shuntPower > 5){
            createNode("Peak power dissipation in the shunt resistor is " + Math.round(shuntPower) + 
            " W. Please choose appropriate power rating of the shunt resistor.", tiRed);
        }
        
        //Compare opamp GBW with signal freq
        //Why 0.142? As bw is -3dB, 0.142*bw is -1% point wrt DC gain or -0.08dB
        if(sineWaveRadio.checked && maxFreq > (0.142 * bw) && minGBWForSine <= 300){
            createNode("Gain bandwidth product (GBW) of the OpAmp is insufficient to meet the design criteria. Please choose an OpAmp with higher GBW specification. The required GBW for the design is "
            + minGBWForSine + " MHz, and the required slew rate is "+ minSlewRate + " V/µs.", tiRed);
        }
        
        //Supply voltage violation rules
        if((posSupplyVolts - negSupplyVolts > supplyVoltsMax) || (posSupplyVolts - negSupplyVolts < supplyVoltsMin)){
            createNode("Total supply voltage, (V+)-(V-) is outside the OpAmp's recommended supply voltage range.", tiRed);
        }
        
        //High tRise warning
        //Rise/Fall time is more than 30% of ON time of switching wave. Please choose a high GBW opamp and/or high slew rate opamp.
        if(switchingWaveRadio.checked && (minDutyCycle/100)*(1/maxFreq) * 0.3 < settlingTime/1000000 && minGBWForSwitching <= 300){
            if(isOpampGBWLimited){
                createNode("The computed settling time (1%) is higher than minimum on time requirement. Please choose an OpAmp with higher GBW specification. The required gain bandwidth product (GBW) for the design is " + minGBWForSwitching + " MHz.", tiRed);
            }
            else{
                createNode("The computed settling time (1%) is higher than minimum on time requirement. Please choose an OpAmp with higher slew rate specification.", 
                tiRed);
            }
        }
        
        //Min specified gain violation suggestion
        //Noise gain = Acl + 1 = 1 + R2/R1
        if(1 + closedLoopGain < minSpecGain){
            createNode("The closed loop gain is less than the minimum stable gain specification of the OpAmp. Please refer -", tiRed);
            const link = document.createElement("a");
            const textNode = document.createTextNode("Decompensated Amplifier Stabilization Circuit");
            link.appendChild(textNode);
            link.href = "https://www.ti.com/lit/an/sboa528/sboa528.pdf";
            link.target ="_blank";
            link.style.fontSize = "12px";
            warningsContainer.appendChild(link);
        }
        
        //Insufficient slew rate warning for output voltage, only for sine wave
        if(sineWaveRadio.checked && ((slewRateTyp * 1000000) < (Math.PI) * (maxVout - minVout) * maxFreq)){
            createNode("The slew rate of the OpAmp is insufficient to meet the desired output swing at " + frequencyInput.value + 
            " Hz. Please choose an OpAmp with higher slew rate specification.", tiRed);
        }
        
        //Suggestion if feedback resistors are > 10k ohms, that would cause stability issues due to delay in feedback path
        if(r2UserValue != undefined && r2UserValue > 10) {
            createNode("Using high values of feedback resistor (R2) will cause stability issues in the design. Please consider using feedback resistor (R2) with values lower than 10 kΩ.", tiRed);
        }
        
    }
    
    //asynchronous function to load the OPAMP parameters when user selects from drop down list and clicks design circuit button
    async function getGPNInfo(gpn, closedLoopGain) {
        var gpnExtracted = gpn.split(" ",1); //extract the gpn from the selected opamp in drop down list
        const myJson = await getJsonResponse(gpnExtracted[0]);
        console.log("API json response: "+myJson);
        
        //Reading the Json file 
        gbwTyp = myJson[0].paramList.find(param => param.name === "GBWtyp").value.base[0];
        slewRateTyp = myJson[0].paramList.find(param => param.name === "Slew ratetyp").value.base[0];
        //slewRateField.label = slewRateTyp;
        
        var tRiseByGbw = (0.35 * (1 + closedLoopGain)) / gbwTyp; //in seconds, or 2.2 tau = tRise
        tRiseByGbw = tRiseByGbw * 1000000; // in microseconds
        var tRiseBySR = (0.8 * (maxVout - minVout)) / slewRateTyp;
        var maxTrise = Math.max(tRiseByGbw, tRiseBySR);
        var isOpampGBWLimited;
        if(maxTrise == tRiseByGbw){
            isOpampGBWLimited = true;
        }
        else{
            isOpampGBWLimited = false;
        }
        console.log("trise by gbw = " + tRiseByGbw + " tRise by SR = " + tRiseBySR);

        //settling time 1% computed using formula: 2.8Tau + Trise
        //Why trise added? At lower gains settling time will not be accurate
        var settlingTime = ((2.8/(2*3.14))*1000000*(1 + closedLoopGain))/gbwTyp + maxTrise;
        console.log("tau = "+((1/(2*3.14))*1000000*(1 + closedLoopGain))/gbwTyp);
        
        if(switchingWaveRadio.checked){
            if(maxTrise < 0.5){
                riseTimeField.label = (maxTrise * 1000).toFixed(2);
                microSecondLabel2.label = "ns";
            }
            else{
                riseTimeField.label = maxTrise.toFixed(2);
                microSecondLabel2.label = "μs";
            }
            
            if(settlingTime < 0.5){
                settlingTimeField.label = (settlingTime * 1000).toFixed(2);
                microSecondLabel1.label = "ns";
            }
            else{
                settlingTimeField.label = settlingTime.toFixed(2);
                microSecondLabel1.label = "μs";
            }
        }
        else{
            riseTimeField.label = "N/A";
            settlingTimeField.label = "N/A";
        }
        
        //minSpecGain = myJson[0].paramList.find(param => param.name === "Acl, min spec gain").value.base[0];
        
        //Error calculation block
        inputBiasCurrMax = myJson[0].paramList.find(param => param.name === "Input bias currentmax").value.base[0];
        inputOffsetVoltsMax = myJson[0].paramList.find(param => param.name === "Vos (offset voltage at 25°C)max").value.base[0];
        console.log("r1 label = "+r1Field.label);
        console.log("r2 field = "+r2Field.value);
         
        //Error in output voltage cacluation in mV 
        //Max error computation based on Te(shunt tolerance) and Ge(Gain error)
        //Error computed at max load current only
        var vRef = minVout - (minLoadCurrent * rShunt * closedLoopGain);
        var Vo_ideal = maxLoadCurrent * rShunt * closedLoopGain + vRef;
        console.log("Vo_ideal = "+Vo_ideal);
        var r1;
        var r3;
        if(kiloOhmLabel1.label == "kΩ") r1 = Number(r1Field.label)*1000;
        else r1 = Number(r1Field.label);
        var r2 = Number(r2Field.value)*1000;
        if(kiloOhmLabel3.label == "kΩ") r3 = Number(r3Field.label)*1000;
        else r3 = Number(r3Field.label);
        var r4 = Number(r4Field.value)*1000;
        console.log("For error block-");
        console.log("r1 r2 r3 r4 "+r1 + " "+r2+ " "+r3+" "+r4);
        var num = (r4 * r3);
        var den = r4 + r3;
        console.log("num den num/den = "+num+" "+den+" "+(num/den));
        var Req = Number(num)/Number(den); // R4 || R3
        console.log("Vos Req Ib Acl = "+inputOffsetVoltsMax +" "+ Req+" "+inputBiasCurrMax+" "+closedLoopGain);
        //+Vos and +Ib
        /*
        Error voltage = (gain error + shunt tolerance)*(IL * Rsh) + (1+Acl)(Vos + r3||r4 * Ib) - R2*Ib = Vout_nonIdeal - Vout_ideal
        */
        var Vos_ib_error = (Number(inputOffsetVoltsMax) + Number(Req * Number(inputBiasCurrMax))) * Number(1 + closedLoopGain) - Number(r2 * Number(inputBiasCurrMax));
        console.log("Vos_ib_error = "+Vos_ib_error);
        //here shunt tolerance and gain error are assumed equal
        console.log("r1+r3 = "+(r1+r3));
        var Rsh_ = rShunt * (r3 + r4)/(r3 + r4 + rShunt);
        console.log("1+tol = "+(1+tolerance));
        var Vo_non_ideal = maxLoadCurrent * Rsh_ * closedLoopGain * (1 + (tolerance/100)) * (1 + (tolerance/100)) + Math.abs(Vos_ib_error) + vRef;
        console.log("vref Rsh_' = "+vRef+" "+ Rsh_);
        //var Vo_non_ideal = Vo_ideal * (1 + tolerance) * (1 + tolerance) * ()
        console.log("Vo_non_ideal = "+Vo_non_ideal);
        var errorInVoltage = Vo_non_ideal - Vo_ideal;
        console.log("Error in Voltage = "+(errorInVoltage*1000));
        //errorInVoutField.label = (errorInVoltage*1000).toFixed(2); //Error in mV
        
        //Error in current calculation, using error in Vout
        var errorInCurrent = 1000*errorInVoltage/(closedLoopGain*rShunt);
        console.log("Current error = "+errorInCurrent+" "+closedLoopGain+" "+rShunt);
        var errorInPercentFSD = (0.1 * errorInCurrent)/(maxLoadCurrent - minLoadCurrent);
        //Only show 2 significant digits
        if(errorInPercentFSD < 10){
            if(errorInCurrent < 500){
                errorInCurrentUnit.label = "mA (%)";
                errorInCurrentField.label = Math.abs(errorInCurrent.toFixed(1)) + " (" + Math.abs(errorInPercentFSD.toFixed(1))+"%)";
            }
            else{
                var errorInA = errorInCurrent/1000;
                errorInCurrentField.label = Math.abs(errorInA.toFixed(1)) + " (" + Math.abs(errorInPercentFSD.toFixed(1))+"%)";
                errorInCurrentUnit.label = "A (%)";
            }
        }
        else{
            if(errorInCurrent < 500){
                errorInCurrentUnit.label = "mA (%)";
                errorInCurrentField.label = Math.abs(errorInCurrent.toFixed(1)) + " (" + Math.round(errorInPercentFSD) + "%)"; //error in mA
            }
            else{
                var errorInA = errorInCurrent/1000;
                errorInCurrentField.label = Math.abs(errorInA.toFixed(1)) + " (" + Math.round(errorInPercentFSD) + "%)"; //error in mA
                errorInCurrentUnit.label = "A (%)";
            }
        }
        
        //supplyVoltsMax = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)max").value.base[0];
        //supplyVoltsMin = myJson[0].paramList.find(param => param.name === "Total supply voltage (+5 V = 5, ±5 V = 10)min").value.base[0];
        
        //Values not used, UNCOMMENT WHEN REQUIRED
        //outSwingHeadRoomToPos = myJson[0].paramList.find(param => param.name === "Output swing headroom (to positive supply)typ").value.base[0];
        //outSwingHeadRoomToNeg = myJson[0].paramList.find(param => param.name === "Output swing headroom (to negative supply)typ").value.base[0];
        
        //var arr = myJson[0].paramList.find(param => param.name === "Rail-to-rail").value.base;
        //if(arr.length == 2) railToRailField.label = arr[0]+", "+arr[1];
        
        //Shunt power calculation
        var shuntPower = -1;
        if(sineWaveRadio.checked){
            console.log("max min load current = "+maxLoadCurrent+" "+minLoadCurrent);
            shuntPower = Math.max(maxLoadCurrent * maxLoadCurrent * rShunt, minLoadCurrent * minLoadCurrent * rShunt);
            if(shuntPower < 1){
                shuntPowerLossField.label = shuntPower.toFixed(2);
            }
            else if(shuntPower < 10){
                shuntPowerLossField.label = shuntPower.toFixed(1);
            }
            else{
                shuntPowerLossField.label = Math.round(shuntPower);
            }
        }
        else if(switchingWaveRadio.checked) {
            console.log("peakCurrent = "+peakCurrent);
            shuntPower = peakCurrent * peakCurrent * rShunt;
            if(shuntPower < 1){
                shuntPowerLossField.label = shuntPower.toFixed(2);
            }
            else if(shuntPower < 10){
                shuntPowerLossField.label = shuntPower.toFixed(1);
            }
            else{
                shuntPowerLossField.label = Math.round(shuntPower);
            }
        }

        var bw = gbwTyp/(1 + closedLoopGain);
        e2eForumLink.style.display = "block";
        
        //DC transfer characteristics equation
        var k = (1 / (closedLoopGain * rShunt)) * (1 + (rShunt/(r3 + r4)));
        console.log("k ="+k);
        var p = (2 * (tolerance/100)) * k;
        console.log("p = "+p);
        console.log("r2 used for koff caluculation = "+r2);
        //changed to +k from -k
        console.log("Vos = "+inputOffsetVoltsMax);
        console.log("Ib = "+inputBiasCurrMax);
        var koff = k * Number(inputBiasCurrMax) * r2 - k * (Number(inputOffsetVoltsMax) + Req * Number(inputBiasCurrMax)) * (1 + closedLoopGain);
        // var koff =  k * (Number(inputOffsetVoltsMax) + Req * Number(inputBiasCurrMax)) * (1 + closedLoopGain) - k * Number(inputBiasCurrMax) * r2; //-Vos -Ib
        // var koff_ =  k * Number(inputBiasCurrMax) * r2 - k * (Number(inputOffsetVoltsMax) + Req * Number(inputBiasCurrMax)) * (1 + closedLoopGain); //+Vos +Ib
        // var koff__ =  (-1) * k * Number(inputBiasCurrMax) * r2 - k * (Number(inputOffsetVoltsMax) - Req * Number(inputBiasCurrMax)) * (1 + closedLoopGain); //+Vos -Ib
        // var koff___ =  k * Number(inputBiasCurrMax) * r2 - k * ((-1) * Number(inputOffsetVoltsMax) + Req * Number(inputBiasCurrMax)) * (1 + closedLoopGain); //-Vos Ib
        //console.log("koff = "+koff+" koff_ = "+koff_+" koff__ = "+koff__+" koff___"+koff___);
        console.log("koff ="+koff);
        equationFieldNew.innerHTML = "I<sub>LOAD</sub>(A) = " + k.toFixed(4) + " ⋅ (V<sub>OUT</sub> – V<sub>REF</sub>) ± " + p.toFixed(4) + " ⋅ (V<sub>OUT</sub> – V<sub>REF</sub>)" + " ± " + Math.abs(koff.toFixed(4));
        
        //Update plot
        // DC curves with min and max errors
        var xValuesDC = [];
        var yValuesMaxErrorDC = [];
        var yValuesMinErrorDC = [];
        var yValuesIdeal = [];
        var stepSize = (maxVout - minVout)/100;
        for (let x = minVout; x <= maxVout; x += stepSize) {
            xValuesDC.push(x);
            
            var plusPlus = (k * (x - vRef)) + (p * (x - vRef)) + koff;
            var plusMinus = (k * (x - vRef)) + (p * (x - vRef)) - koff;
            var minusPlus = (k * (x - vRef)) - (p * (x - vRef)) + koff;
            var minusMinus = (k * (x - vRef)) - (p * (x - vRef)) - koff;
            var yValueIdeal = k * (x -vRef);
            
            var maxi = Math.max(plusPlus, plusMinus, minusPlus, minusMinus);
            var mini = Math.min(plusPlus, plusMinus, minusPlus, minusMinus);
            // console.log("For x = "+x);
            // console.log("maxi = "+maxi);
            // console.log("mini = "+ mini);
            
            yValuesMaxErrorDC.push(maxi);
            yValuesMinErrorDC.push(mini);
            yValuesIdeal.push(yValueIdeal);
        }

        var minErrorTrace = {
            x: xValuesDC,
            y: yValuesMinErrorDC,
            mode: 'lines + markers',
            name: 'Ideal ± Error',
            type: 'scatter',
            line: {
                dash: 'dot'
            },
            marker: {
                size: 5,
                color: "red"
            }
        };
        
        var maxErrorTrace = {
            x: xValuesDC,
            y: yValuesMaxErrorDC,
            mode: 'lines + markers',
            name: 'Ideal ± Error',
            type: 'scatter',
            line: {
                dash: 'dot'
            },
            marker: {
                size: 5,
                color: "red"
            },
            showlegend: false
        };
        
        var idealDCTrace = {
            y: yValuesIdeal,
            x: xValuesDC,
            mode: 'lines',
            name: 'Ideal curve',
            type: 'line',
            marker: {
                color: 'blue',
                opacity: 0,
                size: 0
            }
        }

        // Sine wave input and output computation
        let exp = "Math.sin(x * Math.PI * 2 * maxFreq)";
        var timePeriod = 1/maxFreq;
        console.log("Time period = "+timePeriod);
        //use modified closed loop gain here
        var ratio = (maxFreq * maxFreq)/((bw * bw));
        console.log("freq ratio = "+ratio);
        var den = Math.sqrt(1 + ratio);
        console.log("Gain is reduced by factor of " + den);
        var xValues = [];
        var yValuesInput = [];
        var yValuesOutputActual = [];
        var yValuesOutputIdeal = [];
        for (let x = 0; x <= (timePeriod + timePeriod/1e20); x += (timePeriod/1e3)) {
            xValues.push(x);
            //Attenuate only the AC component of load current
            //Pass the whole DC component of waveform without attenuation
            var yInput = peakCurrent * eval(exp);
            var yOutputActual = (yInput * rShunt * (closedLoopGain/den)) + (dcLevelInput.value * rShunt * closedLoopGain) + vRef;
            var yOutputIdeal = (yInput * rShunt * closedLoopGain) + (dcLevelInput.value * rShunt * closedLoopGain) + vRef;
            yValuesInput.push(yInput);
            yValuesOutputActual.push(yOutputActual);
            yValuesOutputIdeal.push(yOutputIdeal);
        }
        
        var sineInputTrace = {
            x: xValues,
            y: yValuesInput,
            mode: 'lines + markers',
            name: 'I<sub>LOAD</sub> (y1)',
            type: 'scatter',
            marker: {
                size: 5
            }
        };
        
        var sineActualOutputTrace = {
            x: xValues,
            y: yValuesOutputActual,
            yaxis: 'y2',
            mode: 'lines + markers',
            name: 'Actual V<sub>OUT</sub> (y2)',
            type: 'scatter',
            marker: {
                color: "red",
                size: 5
            }
        };
        
        var sineIdealOutputTrace = {
            x: xValues,
            y: yValuesOutputIdeal,
            yaxis: 'y2',
            mode: 'lines + markers',
            name: 'Ideal V<sub>OUT</sub> (y2)',
            type: 'scatter',
            marker: {
                size: 5
            },
            marker: {
            color: 'green'
            },
            line: {
                dash: 'dot'
            }
        };
        
        //Switching wave input load current waveform
        var middle = (minDutyCycle/100) * timePeriod;
        var switchingInputTrace = {
            x: [0, timePeriod/1e20, middle, middle + (timePeriod/1e20), timePeriod],
            y: [0, peakCurrent, peakCurrent, 0, 0],
            mode: 'lines + markers',
            name: 'I<sub>LOAD</sub> (y1)',
            type: 'scatter',
            marker: {
            opacity: 0,
            size: 0,
            color: "blue"
          }
        };
        
        // Switching wave output computation | First order response
        var tau = settlingTime / (5 * 1000000); //in seconds;
        var xValuesSwitch = [];
        var yValuesOutputActual = [];
        console.log("middle = "+ middle);
        for (let x = 0; x <= middle; x += (timePeriod/1e4)) {
            xValuesSwitch.push(x);
            var temp = (-1 * x) / tau;
            var yOutputActualRise = Number(minVout) + (maxVout - minVout) * (1 - Math.pow(Math.E, temp));
            yValuesOutputActual.push(yOutputActualRise);
        }
        //start fall curve from the end of rise curve
        var temp = (-1 * middle) / tau;
        var middleVal = Number(minVout) + (maxVout - minVout) * (1 - Math.pow(Math.E, temp));
        
        for (let x = 0; x <= (timePeriod - middle); x += (timePeriod/1e4)) {
            var yOutputActualFall = Number(minVout) + ((middleVal - minVout) * Math.exp((-1 * x) / tau));
            yValuesOutputActual.push(yOutputActualFall);
            var newX = x + middle;
            xValuesSwitch.push(newX);
        }

        var switchingOutputActualTrace = {
            x: xValuesSwitch,
            y: yValuesOutputActual,
            yaxis: 'y2',
            mode: 'lines + markers',
            name: 'Actual V<sub>OUT</sub> (y2)',
            type: 'scatter',
            marker: {
                size: 5,
                color: "red"
            }
        };
        
        var switchingOutputIdealTrace = {
            x: [0, timePeriod/1e20, middle, middle + (timePeriod/1e20), timePeriod],
            y: [minVout, maxVout, maxVout, minVout, minVout],
            yaxis: "y2",
            mode: 'lines + markers',
            name: 'Ideal V<sub>OUT</sub> (y2)',
            type: 'scatter',
            marker: {
                opacity: 0,
                size: 0,
                color: "green"
            },
            line: {
                dash: 'dot'
            }
        };
        
        var dataTransientSine = [sineInputTrace, sineActualOutputTrace, sineIdealOutputTrace];
        var dataTransientSwitching = [switchingInputTrace, switchingOutputActualTrace, switchingOutputIdealTrace];
        var dataDC = [idealDCTrace, minErrorTrace, maxErrorTrace];
        
        var layoutDC = { 
            font: {size: 10},
            autosize: true,
            margin: {
            l: 40,
            r: 40,
            b: 30,
            t: 30
            },
            xaxis: {
                title: 'V<sub>OUT</sub> (V)'
            },
            yaxis: {
                title: 'I<sub>LOAD</sub> (A)'
            }
        };
        
        var layoutTransientSine = { 
            font: {size: 10},
            autosize: true,
            margin: {
            l: 40,
            r: 40,
            b: 30,
            t: 30
            },
            xaxis: {
                title: 't (s)'
            },
            yaxis: {
                title: 'I<sub>LOAD</sub> (A)'
            },
            yaxis2: {
                title: 'V<sub>OUT</sub> (V)',
                overlaying: 'y', // Overlay on the first y-axis
                side: 'right',
                autosize: true
            }
        };
        
        var layoutTransientSwitching = { 
            font: {size: 10},
            autosize: true,
            margin: {
            l: 40,
            r: 40,
            b: 30,
            t: 30
            },
            xaxis: {
                title: 'Time (s)'
            },
            yaxis: {
                title: 'I<sub>LOAD</sub> (A)'
            },
            yaxis2: {
                title: 'V<sub>OUT</sub> (V)',
                overlaying: 'y', // Overlay on the first y-axis
                side: 'right',
                autosize: true
            }
        };
        
        Plotly.react("dcPlot", dataDC, layoutDC);
        if(sineWaveRadio.checked){
            Plotly.react("transientPlot", dataTransientSine, layoutTransientSine);
        } 
        else {
            Plotly.react("transientPlot", dataTransientSwitching, layoutTransientSwitching);
        }
        createWarnings(shuntPower, bw, supplyVoltsMax, supplyVoltsMin, settlingTime, minSpecGain, closedLoopGain, isOpampGBWLimited, r2UserValue, r4UserValue);
    }
    
    function checkAllInputFields() {
        if(maxFreq == undefined || (sineWaveRadio.checked && dcLevelInput.value == undefined) || minVout == undefined || maxVout == undefined || 
        negSupplyVolts == undefined || posSupplyVolts == undefined || rShunt == undefined || (switchingWaveRadio.checked && minDutyCycle == undefined)){
            console.log("Some fiels empty!");
            alert("Please provide all the input parameters!");
            return true;
        }
        if(maxVout <= minVout){
            alert("Maximum Vout is less than or equal to minimum output voltage!");
            return true;
        }
        
        if(maxVout > posSupplyVolts || minVout < negSupplyVolts){
            alert("The OpAmp's output swing is not withing the limits (V+) and (V-)!");
            return true;
        }
        
        if(maxFreq == -1){
            alert("Please provide correct unit for maximum frequency!");
            return true;
        }
        
        if(rShunt == -1){
            alert("Please provide correct unit for shunt resistor!");
            return true;
        }
        
        if(r2UserValue != undefined && (isNaN(r2UserValue) || r2UserValue == 0)){
            alert("Please provide a finite / correct value for R2 resistor!");
            return true;
        }
        
        if(r4UserValue != undefined && (isNaN(r4UserValue) || r4UserValue == 0)){
            alert("Please provide a finite / correct value for R4 resistor!");
            return true;
        }
        
        return false;
    }
    
    function compute(){
        takeInputs();
        if(checkAllInputFields()) return;
        
        //loader logic 
        loader.style.display = "block";
        let time1 = performance.now();
        
        //Disable the R2 and R4 inputs fields at the start
        r2Field.disabled = false;
        r4Field.disabled = false;
        console.log("compute called with gpn = " + opamp);
        
        //closed loop gain 
        var num = maxVout - minVout;
        var den;
        if(switchingWaveRadio.checked){
            den = maxLoadCurrent * rShunt;
        }
        else {
            den = (2 * peakCurrent) * rShunt;
        } 
        var closedLoopGain = num / den;
        closedLoopGainField.label = closedLoopGain.toFixed(2);
        
        if(r2UserValue == undefined){
            if(closedLoopGain <= 10){
                r1Field.label = 1;
                r2Field.value = closedLoopGain.toFixed(2);
                kiloOhmLabel1.label = "kΩ";
            }
            else {
                var r1 = (10/closedLoopGain) * 1e3;
                var r1 = Math.round(r1);
                kiloOhmLabel1.label = "Ω";
                r1Field.label = r1;
                r2Field.value = 10;
            }
        }
        
        if(r4UserValue == undefined){
            if(closedLoopGain <= 10){
                r3Field.label = 1;
                r4Field.value = closedLoopGain.toFixed(2);
                kiloOhmLabel3.label = "kΩ";
            }
            else {
                var r3 = (10/closedLoopGain) * 1e3;
                var r3 = Math.round(r3);
                kiloOhmLabel3.label = "Ω";
                r3Field.label = r3;
                r4Field.value = 10;
            }
        }
        
        else{
            //Call change() agian to cover an edge case where user changed the values and then changes the Acl and then clicks compute. 
            //Then the corresponding resistor values r1 and r3 should change according to the new Acl
            changeR2();
            changeR4();
            r1Field.label = r1UserValue;
            r3Field.label = r3UserValue;
            r2Field.value = r2UserValue;
            r4Field.value = r4UserValue;
        }
        
        console.log("r1UserValue = "+r1UserValue);
        console.log("r2UserValue = "+r2UserValue);
        console.log("r3UserValue = "+r3UserValue);
        console.log("r4UserValue = "+r4UserValue);
        
        //Vref caluculation from closed loop gain and other inputs
        console.log("minVout = "+minVout);
        console.log("Rsh = "+rShunt);
        console.log("Acl = "+closedLoopGain);
        console
        var vRef = minVout - (minLoadCurrent * rShunt * closedLoopGain);
        vrefField.label = vRef.toFixed(2);
        
        getGPNInfo(opamp, closedLoopGain);
        
        //loader logic for API call only
        let time2 = performance.now();
        setTimeout(function() {
            loader.style.display = "none";
        }, (time2-time1)*1000);
    }
    
    function changeR2(){
        console.log("R2 changed!");
        //closed loop gain
        var num = maxVout - minVout;
        var den = (maxLoadCurrent - minLoadCurrent) * rShunt;
        var closedLoopGain = num / den;
        var r2 = r2Field.value;
        var r1 = r2 / closedLoopGain;
        if(r1 >= 10) {
            r1 = r1.toFixed(1);
            kiloOhmLabel1.label = "kΩ";
        }
        else if(r1 < 10 && r1 >= 1) {
            r1 = r1.toFixed(2);
            kiloOhmLabel1.label = "kΩ";
        }
        
        else if (r1 < 1) {
            r1 = r1 * 1e3;
            r1 = Math.round(r1);
            kiloOhmLabel1.label = "Ω";
        }
        r1Field.label = r1;
        r1UserValue = r1;
        r2UserValue = r2;
    }
    
    function changeR4() {
        console.log("r4 changed");
        //closed loop gain
        var num = maxVout - minVout;
        var den = (maxLoadCurrent - minLoadCurrent) * rShunt;
        var closedLoopGain = num / den;
        var r4 = r4Field.value;
        var r3 =  r4 / closedLoopGain;
        if(r3 >= 10) {
            r3 = r3.toFixed(1);
            kiloOhmLabel3.label = "kΩ";
        }
        else if(r3 < 10 && r3 >= 1) {
            r3 = r3.toFixed(2);
            kiloOhmLabel3.label = "kΩ";
        }
        
        else if (r3 < 1) {
            r3 = r3 * 1e3;
            r3 = Math.round(r3);
            kiloOhmLabel3.label = "Ω";
        }
        r3Field.label = r3;
        r4UserValue = r4;
        r3UserValue = r3;
    }
    
    //compute function to calculate and show the results
    designCircuitButton.addEventListener('click', compute);
    
    //function to reset all inputs with default values
    function resetInputs() {
        minimumDutyCycleInput.value = 50;
        dcLevelInput.value = 0;
        amplitudeInput.value = 10;
        frequencyInput.value = "100k";
        positiveSupplyVoltageInput.value = 5;
        negativeSupplyVoltageInput.value = 0;
        maximumVoutInput.value = 3.3;
        minimumVoutInput.value = 0;
        shuntResistanceInput.value = "10m";
        toleranceSelectDropDown.value = "0.1";
        opampSelectDropDown.selectedIndex = 17;
        updateOpampSpecsTable();
    }
    
    resetButton.addEventListener("click", resetInputs);

//Script ends



/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for registering menu and toolbar action callback
-------------------------------------------------------------------------------------------------------------------------------
**/

ActionRegistry.registerAction('cmd_exit', {
    run() { GcUtils.isNW ? require('nw.gui').Window.get().close() : window.close(); }
});